CREATE OR REPLACE PACKAGE BODY BPMMGR.PROCESS_XML_XX_APP
IS

  FUNCTION context_header(
    p_key IN VARCHAR2
  , p_xml_params IN XMLTYPE DEFAULT NULL
  ) RETURN XMLTYPE IS
    
    l_xml_data XMLTYPE;
    
  BEGIN
  
    SELECT 
      xmlelement("CONTEXT"
      , xmlelement("SUBJECT"
        , xmlelement("VALUE", pa.application_reference)
        , xmlelement("DESC", 'Application Reference')
        , xmlelement("USE", xmlelement("PURPOSE", 'A1'))
        )
      , xmlelement("SUBJECT"
        , xmlelement("VALUE", pat.title)
        , xmlelement("DESC", 'Application Type')
        , xmlelement("USE", xmlelement("PURPOSE", 'B1'))
        )
      , xmlelement("SUBJECT"
        , xmlelement("VALUE", pad.status)
        , xmlelement("DESC", 'Status')
        , xmlelement("USE", xmlelement("PURPOSE", 'C1'))
        )
    )
    INTO l_xml_data
    FROM trainingmgr.xx_plan_apps pa
    JOIN trainingmgr.xx_plan_app_details pad ON pad.application_id = pa.id
    JOIN trainingmgr.xx_plan_app_types pat ON pat.id = pa.type_id
    WHERE pa.id = p_key
    AND pad.status_control = 'C';
    
    RETURN l_xml_data;
  
  END context_header;
  
  
  FUNCTION workbasket(
    p_key VARCHAR2
  , p_xml_params XMLTYPE DEFAULT NULL
  ) RETURN XMLTYPE
  IS
    l_xml_data XMLTYPE;
  BEGIN
  
    SELECT
      xmlelement("WORKBASKET"
      , xmlelement("FOLDER_NUMBER", pa.application_reference)
      , xmlelement("SUBJECT", 'Planning Application')
      , xmlelement("TOPIC", pat.title)
      , xmlelement("STATUS", pad.status)
      )
    INTO l_xml_data
    FROM trainingmgr.xx_plan_apps pa
    JOIN trainingmgr.xx_plan_app_details pad ON pad.application_id = pa.id
    JOIN trainingmgr.xx_plan_app_types pat ON pat.id = pa.type_id
    WHERE pa.id = p_key
    AND pad.status_control = 'C';

    RETURN l_xml_data;
    
  END workbasket;

  FUNCTION email_subject_prefix(
    p_key IN VARCHAR2
  , p_xml_params IN XMLTYPE DEFAULT NULL
  ) RETURN VARCHAR2
  IS
    l_subject_prefix VARCHAR2(100);
  BEGIN
  
    SELECT 'Planning Application (' || pa.application_reference || ')'
    INTO l_subject_prefix
    FROM trainingmgr.xx_plan_apps pa
    WHERE pa.id = p_key;
    
    RETURN l_subject_prefix;

  END email_subject_prefix;
  
  FUNCTION get_unassigned_reviewer (
    p_key IN VARCHAR2 
  , p_xml_params IN XMLTYPE DEFAULT NULL 
  ) RETURN NUMBER
  IS
  
    l_person_id NUMBER;
  
  BEGIN
    
    SELECT MIN(rmc.person_id)
    INTO l_person_id
    FROM decmgr.resource_members_current rmc
    WHERE rmc.res_type = 'XX_TRAINING_PLAN_APP_TEAM'
    AND rmc.role_name = 'PLAN_APP_REVIEWER'
    AND rmc.person_id || 'PER' NOT IN (
      SELECT xbc.secondary_data_uref
      FROM bpmmgr.xview_business_contexts xbc
      JOIN bpmmgr.business_routine_contexts brc ON xbc.bc_id = brc.bc_id
      JOIN bpmmgr.business_stages bs ON brc.id = bs.brc_id
      WHERE xbc.primary_data_uref = p_key || 'XX_APP'
      AND bs.end_datetime IS NULL
      AND bs.stage_label = 'PLNAPPRVW3'
    );
    
    RETURN l_person_id;
  
  END get_unassigned_reviewer;
  
  FUNCTION count_assigned_reviewers (
    p_key IN VARCHAR2
  , p_xml_params IN XMLTYPE DEFAULT NULL
  ) RETURN NUMBER
  IS
  
    l_count NUMBER;
    
  BEGIN
  
    SELECT COUNT(rmc.person_id)
    INTO l_count
    FROM decmgr.resource_members_current rmc
    WHERE rmc.res_type = 'XX_TRAINING_PLAN_APP_TEAM'
    AND rmc.role_name = 'PLAN_APP_REVIEWER'
    AND rmc.person_id || 'PER' IN (
      SELECT xbc.secondary_data_uref
      FROM bpmmgr.xview_business_contexts xbc
      JOIN bpmmgr.business_routine_contexts brc ON xbc.bc_id = brc.bc_id
      JOIN bpmmgr.business_stages bs ON brc.id = bs.brc_id
      WHERE xbc.primary_data_uref = p_key || 'XX_APP'
      AND bs.end_datetime IS NULL
      AND bs.stage_label = 'PLNAPPRVW3'
    );
    
    RETURN l_count;
    
  END count_assigned_reviewers;
  
  FUNCTION count_unassigned_reviewers (
    p_key IN VARCHAR2
  , p_xml_params IN XMLTYPE DEFAULT NULL
  ) RETURN NUMBER
  IS
  
    l_count NUMBER;
    
  BEGIN
  
    SELECT COUNT(rmc.person_id)
    INTO l_count
    FROM decmgr.resource_members_current rmc
    WHERE rmc.res_type = 'XX_TRAINING_PLAN_APP_TEAM'
    AND rmc.role_name = 'PLAN_APP_REVIEWER'
    AND rmc.person_id || 'PER' NOT IN (
      SELECT xbc.secondary_data_uref
      FROM bpmmgr.xview_business_contexts xbc
      JOIN bpmmgr.business_routine_contexts brc ON xbc.bc_id = brc.bc_id
      JOIN bpmmgr.business_stages bs ON brc.id = bs.brc_id
      WHERE xbc.primary_data_uref = p_key || 'XX_APP'
      AND bs.end_datetime IS NULL
      AND bs.stage_label = 'PLNAPPRVW3'
    );
    
    RETURN l_count;
    
  END count_unassigned_reviewers;
  
  FUNCTION review_manage_action_desc (
    p_key IN VARCHAR2
  , p_xml_params IN XMLTYPE DEFAULT NULL
  ) RETURN VARCHAR2
  IS
  BEGIN
  
    RETURN CHR(10) || count_assigned_reviewers(p_key) || ' Reviewers Assigned' || CHR(10)
      || count_unassigned_reviewers(p_key) || ' Reviewers Not Assigned';
    
  END review_manage_action_desc;
  
END process_xml_xx_app;
/
