DECLARE

  l_initials VARCHAR2(3) := UPPER(:initials);
  
  l_res_type VARCHAR2(50);
  l_res_id NUMBER;
  l_rp_id NUMBER;
  l_email_address VARCHAR2(300);
  l_status_code VARCHAR2(4000);
  l_status_message VARCHAR2(4000);
  l_new_wua_id NUMBER;
  
BEGIN

  /* check initials are valid */
  IF NOT(LENGTH(l_initials) BETWEEN 2 AND 3) THEN
    raise_application_error(-20999, 'Your initials must be between two and three characters long.');
  END IF;
  
  /* create UREF type */
  INSERT INTO bpmmgr.uref_types (
    uref_type
  , object_owner
  , object_name
  , object_pk_1_column_name
  , uref_fk1_column_name
  , description
  , expanded_type
  , internal_name
  )
  VALUES (
    l_initials || 'APP'
  , 'TRAININGMGR'
  , l_initials || '_PLAN_APPS'
  , 'ID'
  , l_initials || '_PLAN_APPS_ID'
  , 'Uref for planning applications'
  , l_initials || '_PLAN_APPS'
  , l_initials || '_PLAN_APPS'
  );
  
  EXECUTE IMMEDIATE 'GRANT SELECT, REFERENCES, INDEX ON trainingmgr.' || l_initials || '_plan_apps TO bpmmgr';
  
  bpmmgr.uref_types_auto.command (
    p_uref_type => l_initials || 'APP'
  , p_command   => 'C'
  );
  
  l_res_type := l_initials || '_TRAINING_PLAN_APP_TEAM';
  
  /* create team */
  l_res_id := decmgr.contact.create_default_team(
    p_resource_type => l_res_type
  , p_resource_name => l_initials || ' Planning Application Team'
  , p_resource_desc => l_initials || ' Planning Application Team'
  , p_creating_wua_id => 1
  );
  
  /* for each role create a person account, wua account and add person to team in selected role */
  FOR role_rec IN (
    SELECT rtr.role_name
    FROM decmgr.xview_resource_type_roles rtr
    WHERE rtr.res_type = l_res_type
  )
  LOOP
    
    /* create 2 users for each role */
    FOR i IN 1..2 LOOP
          
      l_email_address := role_rec.role_name || '.' || i || '@' || LOWER(l_initials) || '-plan-app.com';
              
      l_rp_id := securemgr.xcontact.person_id(
        p_email => l_email_address
      , p_forename => l_initials
      , p_surname => role_rec.role_name || i
      , p_tel => NULL
      , p_creating_wua_id => 1
      );
        
      decmgr.contact.add_member(
        p_res_id => l_res_id
      , p_role_name => role_rec.role_name
      , p_person_id => l_rp_id
      , p_requesting_wua_id => 1
      );
        
      dbms_output.put_line('Added user ' || l_email_address || ' to ' || l_res_type || ' in role ' || role_rec.role_name);
        
    END LOOP;
    
  END LOOP;

  COMMIT;
  
END; 
/

/* get login_ids and role names for team members */
SELECT wuac.login_id, rmc.role_name
FROM securemgr.web_user_account_histories wuac
JOIN decmgr.resource_members_current rmc ON wuac.wua_id = rmc.wua_id
WHERE rmc.res_type = :initials || '_TRAINING_PLAN_APP_TEAM'
/
