-- Table for the different types of planning applications
CREATE TABLE trainingmgr.xx_plan_app_types (
  id              NUMBER(6)       NOT NULL
, title           VARCHAR2(40)    NOT NULL
, description     VARCHAR2(4000)  NOT NULL
, price           NUMBER(6,2)     NOT NULL
, start_datetime  DATE            NOT NULL
, end_datetime    DATE
)
TABLESPACE tbsdata;
/

ALTER TABLE trainingmgr.xx_plan_app_types ADD
CONSTRAINT xx_plan_app_types_pk
PRIMARY KEY (id)
USING INDEX TABLESPACE tbsidx;
/

ALTER TABLE trainingmgr.xx_plan_app_types ADD
CONSTRAINT xx_plan_app_types_ck1 CHECK (start_datetime <= end_datetime);
/

-- Master table for planning applications
CREATE TABLE trainingmgr.xx_plan_apps (
  id                     NUMBER(6)     NOT NULL
, type_id                NUMBER(6)     NOT NULL
, application_reference  VARCHAR2(20)  NOT NULL
)
TABLESPACE tbsdata
/

ALTER TABLE trainingmgr.xx_plan_apps ADD
CONSTRAINT xx_plan_apps_pk 
PRIMARY KEY (id)
USING INDEX TABLESPACE tbsidx;
/

ALTER TABLE trainingmgr.xx_plan_apps ADD
CONSTRAINT xx_plan_apps_fk1
FOREIGN KEY (type_id)
REFERENCES trainingmgr.xx_plan_app_types(id);
/

CREATE INDEX trainingmgr.xx_plan_apps_fk1_idx
ON trainingmgr.xx_plan_apps(type_id)
TABLESPACE tbsidx;
/

CREATE TABLE trainingmgr.xx_plan_app_details (
  id              NUMBER(6)    NOT NULL
, application_id  NUMBER(6)    NOT NULL
, start_datetime  DATE         NOT NULL
, end_datetime    DATE
, status_control  VARCHAR2(1)
, status          VARCHAR2(40) NOT NULL
, xml_data        XMLTYPE      NOT NULL
)
TABLESPACE tbsdata
XMLTYPE COLUMN xml_data STORE AS CLOB xx_plan_app_details_xml (TABLESPACE tbsclob ENABLE STORAGE IN ROW CACHE);
/

ALTER TABLE trainingmgr.xx_plan_app_details ADD
CONSTRAINT xx_plan_app_details_pk 
PRIMARY KEY (id)
USING INDEX TABLESPACE tbsidx;
/

CREATE INDEX trainingmgr.xx_plan_app_details_idx1
ON trainingmgr.xx_plan_app_details(status_control)
TABLESPACE tbsidx;
/

ALTER TABLE trainingmgr.xx_plan_app_details ADD
CONSTRAINT xx_plan_app_details_fk1
FOREIGN KEY (application_id)
REFERENCES trainingmgr.xx_plan_apps(id);
/

CREATE INDEX trainingmgr.xx_plan_app_details_fk1_idx
ON trainingmgr.xx_plan_app_details(application_id)
TABLESPACE tbsidx;
/

ALTER TABLE trainingmgr.xx_plan_app_details ADD
CONSTRAINT xx_plan_app_details_ck1 CHECK (start_datetime <= end_datetime);
/

ALTER TABLE trainingmgr.xx_plan_app_details ADD
CONSTRAINT xx_plan_app_details_ck2 CHECK (status_control IN ('C', NULL));
/

CREATE SEQUENCE trainingmgr.xx_plan_app_types_seq
MINVALUE 1
START WITH 1
INCREMENT BY 1;
/

CREATE SEQUENCE trainingmgr.xx_plan_apps_seq
MINVALUE 1
START WITH 1
INCREMENT BY 1;
/

CREATE SEQUENCE trainingmgr.xx_plan_app_details_seq
MINVALUE 1
START WITH 1
INCREMENT BY 1;
/

GRANT SELECT, INSERT, UPDATE ON trainingmgr.xx_plan_app_types TO appenv;
/

GRANT SELECT, INSERT ON trainingmgr.xx_plan_apps TO appenv;
/

GRANT SELECT, INSERT, UPDATE ON trainingmgr.xx_plan_app_details TO appenv;
/

CREATE VIEW trainingmgr.xview_xx_plan_app_details AS (SELECT 1 x FROM DUAL);
/

GRANT SELECT ON trainingmgr.xview_xx_plan_app_details TO appenv;
/

CREATE TABLE trainingmgr.xx_plan_app_docs (
  file_id           VARCHAR2(30)  NOT NULL
, file_description  VARCHAR2(4000)
, file_content      BLOB          NOT NULL
, created_date      DATE
, created_by        NUMBER(20)
)
TABLESPACE tbsdata
LOB(file_content) STORE AS (TABLESPACE tbsblob ENABLE STORAGE IN ROW CACHE)
/